

#ifndef TNT_ARRAY3D_UTILS_H
#define TNT_ARRAY3D_UTILS_H

#include <cstdlib>
#include <cassert>

namespace TNT
{


template <class T>
std::ostream& operator<<(std::ostream &s, const Array3D<T> &A)
{
    int M=A.dim1();
    int N=A.dim2();
    int K=A.dim3();

    s << M << " " << N << " " << K << "\n";

    for (int i=0; i<M; i++)
    {
        for (int j=0; j<N; j++)
        {
			for (int k=0; k<K; k++)
            	s << A[i][j][k] << " ";
			s << "\n";
        }
        s << "\n";
    }


    return s;
}

template <class T>
std::istream& operator>>(std::istream &s, Array3D<T> &A)
{

    int M, N, K;

    s >> M >> N >> K;

	Array3D<T> B(M,N,K);

    for (int i=0; i<M; i++)
        for (int j=0; j<N; j++)
			for (int k=0; k<K; k++)
            	s >>  B[i][j][k];

	A = B;
    return s;
}



template <class T>
Array3D<T> operator+(const Array3D<T> &A, const Array3D<T> &B){
	int m = min(A.dim1(), B.dim1());
	int n = min(A.dim2(), B.dim2());
	int p = min(A.dim3(), B.dim3());
	Array3D<T> C(m,n,p);
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				C[i][j][k] = A[i][j][k] + B[i][j][k];
	return C;
	}

template <class T>
Array3D<T> operator-(const Array3D<T> &A, const Array3D<T> &B){
	int m = min(A.dim1(), B.dim1());
	int n = min(A.dim2(), B.dim2());
	int p = min(A.dim3(), B.dim3());
	Array3D<T> C(m,n,p);
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				C[i][j][k] = A[i][j][k] - B[i][j][k];
	return C;
	}

template <class T>
Array3D<T> operator*(const Array3D<T> &A, const Array3D<T> &B){
	int m = min(A.dim1(), B.dim1());
	int n = min(A.dim2(), B.dim2());
	int p = min(A.dim3(), B.dim3());
	Array3D<T> C(m,n,p);
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				C[i][j][k] = A[i][j][k] * B[i][j][k];
	return C;
	}

template <class T>
Array3D<T> operator/(const Array3D<T> &A, const Array3D<T> &B){
	int m = min(A.dim1(), B.dim1());
	int n = min(A.dim2(), B.dim2());
	int p = min(A.dim3(), B.dim3());
	Array3D<T> C(m,n,p);
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				C[i][j][k] = B[i][j][k]==0 ? 0 : (A[i][j][k]/B[i][j][k]);
	return C;
	}

//------------------------- modified - if not same size, calculate as much as possible
template <class T>
Array3D<T>& operator+=(Array3D<T> &A, const Array3D<T> &B){
	int m = min( A.dim1(), B.dim1() );
	int n = min( A.dim2(), B.dim2() );
	int p = min( A.dim3(), B.dim3() );
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				A[i][j][k] += B[i][j][k];
	return A;
	}

template <class T>
Array3D<T>& operator-=(Array3D<T> &A, const Array3D<T> &B){
	int m = min( A.dim1(), B.dim1() );
	int n = min( A.dim2(), B.dim2() );
	int p = min( A.dim3(), B.dim3() );
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				A[i][j][k] -= B[i][j][k];
	return A;
	}
template <class T>
Array3D<T>& operator*=(Array3D<T> &A, const Array3D<T> &B){
	int m = min( A.dim1(), B.dim1() );
	int n = min( A.dim2(), B.dim2() );
	int p = min( A.dim3(), B.dim3() );
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				A[i][j][k] *= B[i][j][k];
	return A;
	}

template <class T>
Array3D<T>& operator/=(Array3D<T> &A, const Array3D<T> &B){
	int m = min( A.dim1(), B.dim1() );
	int n = min( A.dim2(), B.dim2() );
	int p = min( A.dim3(), B.dim3() );
	for( int i=0; i<m; i++ )
		for( int j=0; j<n; j++ )
			for( int k=0; k<p; k++ )
				if( B[i][j][k] != 0 ) 
					A[i][j][k] /= B[i][j][k];
	return A;
	}

#ifdef _DEBUG
template <class T>
void OutputDebugArray( Array3D<T> & a, const char * pchName ) { 
	char buffer[2048];
	sprintf( buffer, "%s[%d][%d][%d]=\n", pchName, a.dim1(), a.dim2(), a.dim3() );
	OutputDebugString( buffer );
	for( int i=0; i<a.dim1() && i<10; ++i ) {
		for( int j=0; j<a.dim2() && j<10; ++j ) {
			sprintf( buffer, "[%d][%d]\t{", i, j );
			for( int k=0; k<a.dim3() && k<10 && strlen(buffer)<2000; ++k )
				sprintf( buffer+strlen(buffer), " %.9f", double( a[i][j][k] ) );
			sprintf( buffer+strlen(buffer), "}\n");
			OutputDebugString( buffer );
			}
		}
	}

#endif



} // namespace TNT

#endif
